//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#pragma once

#include "header.h"

// NetBSD の <sys/exec_aout.h>、<sys/aout_mids.h> あたり参考。

#define AOUT_MID(x)		(((x) >> 16) & 0x3ff)
// UniOS-Mach の a.out の machine id はこの辺りで定義されてるやつらしい
// http://www.cs.cmu.edu/afs/cs/project/mach/public/src/mkernel/src/kernel/luna88k/exec.h
#define AOUT_MID_MACH_M88K		(0x04)
#define AOUT_MID_BSD_M68K8K		(0x87)	// 8K page sizes
#define AOUT_MID_BSD_M68K4K		(0x88)	// 4K page sizes
#define AOUT_MID_BSD_M88K		(0x99)

#define AOUT_OMAGIC		(0x0107)

struct aout_header {
	// magic はネットワークバイトオーダ。
	//
	//     3                   2                   1                   0
	//   1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
	// +------------+-------------------+-------------------------------+
	// |    flags   |    machine id     |               magic           |
	// +------------+-------------------+-------------------------------+
	uint32 magic;
	// 以下はホストバイトオーダ。
	uint32 textsize;
	uint32 datasize;
	uint32 bsssize;
	uint32 symsize;
	uint32 entry;
	uint32 textrelsize;
	uint32 datarelsize;
};
