//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#pragma once

#include "config.h"
#include "debugger.h"
#include "logger.h"
#include "vm_luna.h"
#include "vm_x68k.h"

// 起動時の引数処理などを行う。
class MainApp
{
 public:
	// Init() の戻り値
	static const int PASS = -1;		// 実行を継続

 public:
	// 初期化 (ステージ1)
	int Init1(bool is_cli_, int ac, char *av[]);

	// 初期化 (ステージ2)
	bool Init2();

	std::string SearchFile(const std::string& name) const;

	// vmtype (VMTYPE_*) を返す。
	// (Config が持っているのは文字列)
	vmtype_t GetVMType() const { return vmtype; }

	// 現在の VM が指定の feature を持っているか?
	bool HasVMFeature(vmf_t flag) const;

	// ログレベルを設定する。
	static bool SetLogopt(const std::vector<std::string>&, std::string *errmsg);

	// ログ名の一覧を表示する
	static std::vector<std::string> GetLogNames();

 public:
	// -b: ブレークポイント "<addr>[,<skip>]"
	std::vector<std::string> debug_breakaddr {};

	// -B: ベンチマークモード
	int benchmark_mode {};

	// -c: VM ディレクトリ or 設定ファイル
	std::string vmdir {};
	std::string vmfile {};
	const std::string& GetVMDir() const { return vmdir; }

	// -C: ログをコンソールに出力
	bool log_to_console {};

	// -d: 起動時にデバッガで停止
	bool debug_on_start {};

	// -D: コンソールをデバッガで使う
	bool debug_on_console {};

	// -f: 高速モード (XXX 設定ファイルと含めて見直すこと)
	bool fast_mode {};

	// --fontsize: フォントサイズ指定値 (値は fontsize_t ではない)
	int  fontsize {};

	// -H: 内蔵 Human68k 風を組み込む。-X による実行ファイル名が必要。
	bool human_mode {};

	// -L: ログ指定文字列
	char logopt[256] {};

	// -M: モニタ名
	std::string monitor_opt {};

	// -s,--scale: 画面スケール
	double screen_scale {};

	// --show-config: 設定内容を表示 (1なら通常、2ならall)
	int show_config {};

	// -V: パラメータ指定
	std::vector<std::string> config_options {};

	// -X: ホスト実行ファイル名とその引数
	const char *exec_file {};
	std::string exec_arg {};

 private:
	// ヘルプメッセージを表示する
	void ShowHelp(bool all) const;

	// バージョンを表示する
	void ShowVersion() const;

	// 引数を処理する
	bool ParseOpt(int ac, char *av[]);

	// -L オプションを処理する
	bool ParseLogopt();

	// ログレベル1つを設定する。
	static bool SetLogopt1(const std::string&, std::string *errmsg);

	// コンパイルされている host netdriver の一覧を表示
	void ShowHostnet() const;

	// CLI/GUI 区別
	bool IsCLI() const { return  is_cli; }
	bool IsGUI() const { return !is_cli; }
	bool is_cli {};

	// VM 種別
	vmtype_t vmtype {};
};

// グローバルインスタンス
extern MainApp gMainApp;
