//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#pragma once

#if !defined(_LITTLE_ENDIAN)
# define _LITTLE_ENDIAN	1234
# define _BIG_ENDIAN	4321
#endif

#if !defined(_BYTE_ORDER)
# if defined(WORDS_BIGENDIAN)
#  define _BYTE_ORDER _BIG_ENDIAN
# else
#  define _BYTE_ORDER _LITTLE_ENDIAN
# endif
#endif

// __builtin_bswap* がないような環境があったらその時考える
#if !defined(HAVE___BUILTIN_BSWAP16) || \
    !defined(HAVE___BUILTIN_BSWAP32) || \
    !defined(HAVE___BUILTIN_BSWAP64)
#error No __builtin_bswap*
#endif

#define bswap16(x)	__builtin_bswap16(x)
#define bswap32(x)	__builtin_bswap32(x)
#define bswap64(x)	__builtin_bswap64(x)

#if _BYTE_ORDER == _BIG_ENDIAN
#define be16toh(x)	((uint16)(x))
#define be32toh(x)	((uint32)(x))
#define be64toh(x)	((uint64)(x))
#define le16toh(x)	bswap16((uint16)(x))
#define le32toh(x)	bswap32((uint32)(x))
#define le64toh(x)	bswap64((uint64)(x))
#else
#define be16toh(x)	bswap16((uint16)(x))
#define be32toh(x)	bswap32((uint32)(x))
#define be64toh(x)	bswap64((uint64)(x))
#define le16toh(x)	((uint16)(x))
#define le32toh(x)	((uint32)(x))
#define le64toh(x)	((uint64)(x))
#endif

#define htobe16(x)	be16toh(x)
#define htobe32(x)	be32toh(x)
#define htobe64(x)	be64toh(x)

#define htole16(x)	le16toh(x)
#define htole32(x)	le32toh(x)
#define htole64(x)	le64toh(x)
