//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#pragma once

#include "rom.h"
#include "stopwatch.h"

class PROMDevice;

class PROM0Device : public IODevice
{
	using inherited = IODevice;
 public:
	PROM0Device();
	virtual ~PROM0Device() override;

	bool Init() override;
	void ResetHard() override;

	uint64 Read8(uint32 addr) override;
	uint64 Read16(uint32 addr) override;
	uint64 Read32(uint32 addr) override;
	uint64 Write8(uint32 addr, uint32 data) override;
	uint64 Write16(uint32 addr, uint32 data) override;
	uint64 Write32(uint32 addr, uint32 data) override;
	uint64 Peek8(uint32 addr) override;

 private:
	// アドレスデコーダ
	static uint32 Decoder(uint32 addr);

	// 書き込みの共通部分
	uint64 Write();

	IODevice *prom {};
};

class PROMDevice : public ROMDevice
{
	using inherited = ROMDevice;
 public:
	PROMDevice();
	virtual ~PROMDevice() override;

	bool Init() override;
	bool PowerOn() override;

	uint64 Read8(uint32 addr) override;

 private:
	bool InitLuna1();
	bool InitLuna88k();

	// パフォーマンス測定用
	Stopwatch sw {};
};

extern std::unique_ptr<PROM0Device> gPROM0;
extern std::unique_ptr<IODevice> gPROM;
