//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#pragma once

#include "device.h"
#include "mappedfile.h"

class SRAMDevice : public IODevice
{
	using inherited = IODevice;
 private:
	static const uint32 baseaddr = 0xed0000;

	// Pluto-X によるホストファイル起動アドレス
	static const uint32 plutoaddr = 0xeac000;

 public:
	SRAMDevice();
	virtual ~SRAMDevice() override;

	bool Init() override;
	void ResetHard() override;

	uint64 Read8(uint32 addr) override;
	uint64 Read16(uint32 addr) override;
	uint64 Write8(uint32 addr, uint32 data) override;
	uint64 Write16(uint32 addr, uint32 data) override;
	uint64 Peek8(uint32 addr) override;

	void write_enable(bool value);

 private:
	// アドレスデコーダ
	uint64 Decoder(uint32 addr) const;

	// ファイル
	std::string filename {};
	MappedFile file {};
	uint8 *mem {};

	bool writeable {};

	// ホストファイル起動用の読み出し値を返す
	uint32 PeekHostBoot(uint32 offset) const;
};

extern std::unique_ptr<SRAMDevice> gSRAM;
