//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#pragma once

#include "device.h"
#include "monitor.h"
#include "scheduler.h"

class SysClkDevice : public IODevice
{
	using inherited = IODevice;
 public:
	SysClkDevice();
	virtual ~SysClkDevice() override;

	bool Init() override;
	bool PowerOn() override;

 protected:
	// BusIO インタフェース
	static const uint32 NPORT = 1;
	uint64 Read(uint32 offset);
	uint64 Write(uint32 offset, uint32 data);
	uint64 Peek(uint32 offset);

 private:
	// イベントコールバック
	void Callback(Event& ev);
	void Callback_rt(Event& ev);

	DECLARE_MONITOR_CALLBACK(MonitorUpdate);

	// 割り込み状態をレジスタイメージにして返す
	uint8 GetInt() const;

	// システムクロック割り込み有りなら true。
	bool sysint {};

	// 割り込みイベント
	Event event { this };

	// 本来のイベント間隔。機種によって異なる。
	uint period {};

	// システムクロックが刻んでいる時刻
	uint64 stime {};

	// システムクロックを実時間に同期する場合 true
	bool sync_rt {};

	Monitor monitor { this };
};

extern std::unique_ptr<SysClkDevice> gSysClk;
