//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#pragma once

#include "device.h"
#include <atomic>

class TVRAMDevice : public IODevice
{
	using inherited = IODevice;
 private:
	static const int baseaddr = 0xe00000;

	struct tvram {
		bool sa;				// 同時アクセス
		bool ap[4];				// 同時アクセスプレーン
		int xscroll;			// X スクロール
		int yscroll;			// Y スクロール
	};

 public:
	TVRAMDevice();
	virtual ~TVRAMDevice() override;

	void ResetHard() override;

	uint64 Read8(uint32 addr) override;
	uint64 Read16(uint32 addr) override;
	uint64 Write8(uint32 addr, uint32 data) override;
	uint64 Write16(uint32 addr, uint32 data) override;
	uint64 Peek8(uint32 addr) override;

	void set_crtc_21(uint16 data);
	void SetScrollX(int x);
	void SetScrollY(int y);

	bool Render(uint8 *);

 private:
	// アドレスデコーダ
	uint64 Decoder(uint32 addr) const;

	void Invalidate();

	std::unique_ptr<uint8[]> mem {};
	// XXX 実際には縦がこれ以上になる可能性はある
	std::atomic<uint8> atomic_dirty[1024] {};
	struct tvram tvram {};

	// レンダリング時に使う変換テーブル
	static uint32 deptable[256];

	// RGB24 形式(3バイト)を4バイトあるいは SSE なら 16バイト単位で
	// 書き込むための1ライン分の中間バッファ。
	ALIGNAS_IF_X86(16) uint32 rb[768] {};
};

extern std::unique_ptr<TVRAMDevice> gTVRAM;
