//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#include "wxmonitor.h"
#include "wxtextscreen.h"

//
// モニターウィンドウ
//

// コンストラクタ
WXMonitorWindow::WXMonitorWindow(wxWindow *parent, const wxString& name,
	Monitor& monitor_)
	: inherited(parent, wxID_ANY, name)
{
	// top
	// | +----------------------------+
	// v |           上余白           |
	//   |+------+------------+------+|
	//   ||左余白| TextScreen |右余白||
	//   |+------+------------+------+|
	//   |           下余白           |
	//   +----------------------------+

	wxBoxSizer *topsizer = new wxBoxSizer(wxVERTICAL);

	// 上余白
	topsizer->Add(new WXPaddingPanel(this), 0, wxEXPAND);

	// 中段(横sizer)
	wxBoxSizer *hbox = new wxBoxSizer(wxHORIZONTAL);
	topsizer->Add(hbox, 0, wxEXPAND);

	// 左余白
	hbox->Add(new WXPaddingPanel(this), 0, wxEXPAND);

	// テキストスクリーン
	screen = new WXTextScreen(this, monitor_);
	hbox->Add(screen, 0, wxEXPAND);

	// 右余白
	hbox->Add(new WXPaddingPanel(this), 0, wxEXPAND);

	// 下余白
	topsizer->Add(new WXPaddingPanel(this), 0, wxEXPAND);

	SetSizer(topsizer);

	// 大きさをそれに固定
	Fit();
}

// デストラクタ
WXMonitorWindow::~WXMonitorWindow()
{
}
